#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from typing import Optional, Union

import pyrogram
from pyrogram import raw, types


class GetOwnedStarCount:
    async def get_owned_star_count(
        self: "pyrogram.Client",
        user_id: Optional[Union[int, str]] = None,
    ) -> "types.StarAmount":
        """Get the number of Telegram Stars count owned by the current account or the specified bot.

        .. include:: /_includes/usable-by/users.rst

        Parameters:
            user_id (``int`` | ``str``, *optional*):
                Unique identifier (int) or username (str) of the bot for which the star count should be returned instead of the current user.
                The bot should have ``can_be_edited`` property set to True.
                Pass ``None`` to return the count of the current user.

        Returns:
            :obj:`~pyrogram.types.StarAmount`: On success, the current stars balance is returned.

        Example:
            .. code-block:: python

                # Get stars balance
                app.get_stars_balance()

                # Get stars balance of a bot owned by the current user
                app.get_stars_balance(user_id="pyrogrambot")

        """
        if user_id is None:
            peer = raw.types.InputPeerSelf()
        else:
            peer = await self.resolve_peer(user_id)

        r = await self.invoke(
            raw.functions.payments.GetStarsStatus(
                peer=peer
                # TODO
            )
        )

        return types.StarAmount._parse(self, r)


    async def get_business_account_star_balance(
        self: "pyrogram.Client",
        business_connection_id: str,
    ) -> "types.StarAmount":
        """Returns the amount of Telegram Stars owned by a managed business account. Requires the can_view_gifts_and_stars business bot right.

        .. include:: /_includes/usable-by/bots.rst

        Parameters:
            business_connection_id (``str``):
                Unique identifier of the business connection
        
        Returns:
            :obj:`~pyrogram.types.StarAmount`: On success, the current stars balance is returned.
        
        """
        if not business_connection_id:
            raise ValueError("business_connection_id is required")

        business_connection = self.business_user_connection_cache[business_connection_id]
        if business_connection is None:
            business_connection = await self.get_business_connection(business_connection_id)
        r = await self.invoke(
            raw.functions.InvokeWithBusinessConnection(
                query=raw.functions.payments.GetStarsStatus(
                    peer=await self.resolve_peer(
                        business_connection.user_chat_id
                    )
                ),
                connection_id=business_connection_id
            )
        )
        return types.StarAmount._parse(self, r)



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from .alternative_video import AlternativeVideo
from .animation import Animation
from .audio import Audio
from .contact import Contact
from .contact_registered import ContactRegistered
from .dice import Dice
from .document import Document
from .game import Game
from .location import Location
from .message import Message
from .message_entity import MessageEntity
from .photo import Photo
from .poll import Poll
from .poll_answer import PollAnswer
from .poll_option import PollOption
from .reaction import (
    Reaction,
    ReactionType,
    ReactionTypeEmoji,
    ReactionTypeCustomEmoji,
    ReactionTypePaid,
    ReactionCount
)
from .sponsored_message import SponsoredMessage
from .gift import Gift
from .received_gift import ReceivedGift
from .upgraded_gift import UpgradedGift
from .sticker import Sticker
from .stripped_thumbnail import StrippedThumbnail
from .thumbnail import Thumbnail
from .venue import Venue
from .video import Video
from .video_note import VideoNote
from .voice import Voice
from .web_app_data import WebAppData
from .web_page import WebPage
from .message_reactions import MessageReactions
from .message_reaction_updated import MessageReactionUpdated
from .message_reaction_count_updated import MessageReactionCountUpdated
from .chat_boost_added import ChatBoostAdded
from .payment_form import PaymentForm
from .giveaway import Giveaway
from .giveaway_created import GiveawayCreated
from .giveaway_completed import GiveawayCompleted
from .giveaway_winners import GiveawayWinners
from .gift_code import GiftCode
from .gifted_premium import GiftedPremium
from .gifted_stars import GiftedStars
from .message_effect import MessageEffect
from .screenshot_taken import ScreenshotTaken
from .translated_text import TranslatedText
from .message_auto_delete_timer_changed import MessageAutoDeleteTimerChanged
from .write_access_allowed import WriteAccessAllowed
from .paid_message_price_changed import PaidMessagePriceChanged
from .paid_messages_refunded import PaidMessagesRefunded
from .checklist import Checklist
from .checklist_task import ChecklistTask
from .checklist_tasks_added import ChecklistTasksAdded
from .checklist_tasks_done import ChecklistTasksDone
from .direct_message_price_changed import DirectMessagePriceChanged

__all__ = [
    "AlternativeVideo",
    "Animation",
    "Audio",
    "ChatBoostAdded",
    "Contact",
    "ContactRegistered",
    "Dice",
    "Document",
    "Game",
    "PaymentForm",
    "GiftCode",
    "GiftedPremium",
    "GiftedStars",
    "Giveaway",
    "GiveawayCreated",
    "GiveawayCompleted",
    "GiveawayWinners",
    "Location",
    "Message",  # TODO
    "MessageAutoDeleteTimerChanged",
    "MessageEffect",
    "MessageEntity",
    "MessageReactionCountUpdated",
    "MessageReactionUpdated",
    "MessageReactions",
    "Photo",
    "Reaction",
    "ReactionCount",
    "ReactionType",
    "ReactionTypeEmoji",
    "ReactionTypeCustomEmoji",
    "ReactionTypePaid",
    "Thumbnail",
    "StrippedThumbnail",
    "Poll",
    "PollAnswer",
    "PollOption",
    "SponsoredMessage",
    "Gift",
    "ReceivedGift",
    "UpgradedGift",
    "Sticker",
    "Venue",
    "Video",
    "VideoNote",
    "Voice",
    "WebAppData",
    "WebPage",
    "WriteAccessAllowed",
    "ScreenshotTaken",
    "TranslatedText",
    "PaidMessagePriceChanged",
    "PaidMessagesRefunded",
    "Checklist",
    "ChecklistTask",
    "ChecklistTasksAdded",
    "ChecklistTasksDone",
    "DirectMessagePriceChanged",
]



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import logging
from datetime import datetime
from typing import Optional, Union

import pyrogram
from pyrogram import enums, raw, types, utils

log = logging.getLogger(__name__)


class SendChecklist:
    async def send_checklist(
        self: "pyrogram.Client",
        chat_id: Union[int, str],
        checklist: "types.InputChecklist",
        disable_notification: Optional[bool] = None,
        protect_content: Optional[bool] = None,
        message_effect_id: Optional[int] = None,
        message_thread_id: Optional[int] = None,
        reply_parameters: Optional["types.ReplyParameters"] = None,
        schedule_date: Optional[datetime] = None,
        business_connection_id: Optional[str] = None,
        paid_message_star_count: int = None,
        reply_markup: Optional[
            Union[
                "types.InlineKeyboardMarkup",
                "types.ReplyKeyboardMarkup",
                "types.ReplyKeyboardRemove",
                "types.ForceReply"
            ]
        ] = None,
        caption: str = "",
        parse_mode: Optional["enums.ParseMode"] = None,
        caption_entities: list["types.MessageEntity"] = None,
    ) -> "types.Message":
        """Send a new checklist.

        .. include:: /_includes/usable-by/users-bots.rst

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier (int) or username (str) of the target chat.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".
                For a contact that exists in your Telegram address book you can use his phone number (str).

            checklist (:obj:`~pyrogram.types.InputChecklist`):
                Checklist to send.

            disable_notification (``bool``, *optional*):
                Sends the message silently.
                Users will receive a notification with no sound.

            protect_content (``bool``, *optional*):
                Protects the contents of the sent message from forwarding and saving.

            message_effect_id (``int``, *optional*):
                Unique identifier of the message effect.
                For private chats only.

            message_thread_id (``int``, *optional*):
                Unique identifier for the target message thread (topic) of the forum.

            reply_parameters (:obj:`~pyrogram.types.ReplyParameters`, *optional*):
                Describes reply parameters for the message that is being sent.

            schedule_date (:py:obj:`~datetime.datetime`, *optional*):
                Date when the message will be automatically sent.

            business_connection_id (``str``, *optional*):
                Unique identifier of the business connection on behalf of which the message will be sent.

            paid_message_star_count (``int``, *optional*):
                The number of Telegram Stars the user agreed to pay to send the messages.

            reply_markup (:obj:`~pyrogram.types.InlineKeyboardMarkup` | :obj:`~pyrogram.types.ReplyKeyboardMarkup` | :obj:`~pyrogram.types.ReplyKeyboardRemove` | :obj:`~pyrogram.types.ForceReply`, *optional*):
                Additional interface options. An object for an inline keyboard, custom reply keyboard,
                instructions to remove reply keyboard or to force a reply from the user.

            caption (``str``, *optional*):
                Media caption, 0-1024 characters.

            parse_mode (:obj:`~pyrogram.enums.ParseMode`, *optional*):
                By default, texts are parsed using both Markdown and HTML styles.
                You can combine both syntaxes together.

            caption_entities (List of :obj:`~pyrogram.types.MessageEntity`):
                List of special entities that appear in the caption, which can be specified instead of *parse_mode*.

        Returns:
            :obj:`~pyrogram.types.Message`: On success, the sent checklist message is returned.

        Example:
            .. code-block:: python

                await client.send_checklist(
                    chat_id=message.chat.id,
                    checklist=types.InputChecklist(
                        title="To Do",
                        tasks=[
                            types.InputChecklistTask(id=1, text="Task 1"),
                            types.InputChecklistTask(id=2, text="Task 2"),
                            types.InputChecklistTask(id=3, text="Task 3"),
                        ]
                    )
                )
        """
        title, entities = (await utils.parse_text_entities(
            self, checklist.title, checklist.parse_mode, checklist.title_entities
        )).values()

        rpc = raw.functions.messages.SendMedia(
            peer=await self.resolve_peer(chat_id),
            media=raw.types.InputMediaTodo(
                todo=raw.types.TodoList(
                    title=raw.types.TextWithEntities(
                        text=title,
                        entities=entities or []
                    ),
                    list=[await task.write(self) for task in checklist.tasks],
                    others_can_append=checklist.others_can_add_tasks,
                    others_can_complete=checklist.others_can_mark_tasks_as_done
                )
            ),
            silent=disable_notification,
            reply_to=await utils._get_reply_message_parameters(
                self,
                message_thread_id,
                reply_parameters
            ),
            random_id=self.rnd_id(),
            schedule_date=utils.datetime_to_timestamp(schedule_date),
            noforwards=protect_content,
            allow_paid_stars=paid_message_star_count,
            reply_markup=await reply_markup.write(self) if reply_markup else None,
            effect=message_effect_id,
            **await utils.parse_text_entities(self, caption, parse_mode, caption_entities)
        )
        if business_connection_id:
            r = await self.invoke(
                raw.functions.InvokeWithBusinessConnection(
                    query=rpc,
                    connection_id=business_connection_id
                )
            )
        else:
            r = await self.invoke(rpc)

        messages = await utils.parse_messages(
            client=self,
            messages=None,
            business_connection_id=business_connection_id,
            r=r
        )

        return messages[0] if messages else None



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import os
from typing import Callable, Union

import pyrogram
from pyrogram import StopTransmission, enums, raw, types, utils
from pyrogram.errors import FilePartMissing


class PostStory:
    async def post_story(
        self: "pyrogram.Client",
        chat_id: Union[int, str],
        content: "types.InputStoryContent",
        active_period: int = None,
        caption: str = None,
        parse_mode: "enums.ParseMode" = None,
        caption_entities: list["types.MessageEntity"] = None,
        areas: list["types.StoryArea"] = None,
        post_to_chat_page: bool = None,
        protect_content: bool = None,
        business_connection_id: str = None,
        privacy_settings: "types.StoryPrivacySettings" = None,
        from_story_chat_id: Union[int, str] = None,
        from_story_id: int = None,
        progress: Callable = None,
        progress_args: tuple = (),
    ) -> "types.Story":
        """Posts a new story on behalf of a chat.

        .. include:: /_includes/usable-by/users.rst

        Requires can_post_stories right for supergroup and channel chats.

        .. include:: /_includes/usable-by/bots.rst

        Requires the can_manage_stories business bot right.

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier (int) or username (str) of the target chat.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".

            content (:obj:`~pyrogram.types.InputStoryContent`):
                Content of the story.

            active_period (``int``, *optional*):
                Period after which the story is moved to the archive, in seconds; must be one of 6 * 3600, 12 * 3600, 86400, or 2 * 86400.

            caption (``str``, *optional*):
                Caption of the story, 0-2048 characters after entities parsing.
            
            parse_mode (:obj:`~pyrogram.enums.ParseMode`, *optional*):
                By default, texts are parsed using both Markdown and HTML styles.
                You can combine both syntaxes together.

            caption_entities (List of :obj:`~pyrogram.types.MessageEntity`):
                List of special entities that appear in the caption, which can be specified instead of *parse_mode*.

            areas (List of :obj:`~pyrogram.types.StoryArea`, *optional*):
                List of of clickable areas to be shown on the story.

            post_to_chat_page (``bool``, *optional*):
                Pass True to keep the story accessible after it expires.
            
            protect_content (``bool``, *optional*):
                Pass True if the content of the story must be protected from forwarding and screenshotting.

            business_connection_id (``str``):
                Unique identifier of the business connection.

            privacy_settings (:obj:`~pyrogram.types.StoryPrivacySettings`, *optional*):
                The privacy settings for the story; ignored for stories sent to supergroup and channel chats.
                Defaults to :obj:`~pyrogram.types.StoryPrivacySettingsEveryone`.

            from_story_chat_id (``int`` | ``str``, *optional*):
                Full identifier of the original story, which content was used to create the story; pass None if the story isn't repost of another story.
                Identifier of the chat that posted the story.

            from_story_id (``int``, *optional*):
                Full identifier of the original story, which content was used to create the story; pass None if the story isn't repost of another story.
                Unique story identifier among stories of the given sender.

            progress (``Callable``, *optional*):
                Pass a callback function to view the file transmission progress.
                The function must take *(current, total)* as positional arguments (look at Other Parameters below for a
                detailed description) and will be called back each time a new file chunk has been successfully
                transmitted.

            progress_args (``tuple``, *optional*):
                Extra custom arguments for the progress callback function.
                You can pass anything you need to be available in the progress callback scope; for example, a Message
                object or a Client instance in order to edit the message with the updated progress status.

        Returns:
            :obj:`~pyrogram.types.Story` a single story is returned.

        Example:
            .. code-block:: python

                # Post story to your profile
                await app.post_story("me", InputStoryContentPhoto("story.png"), caption='My new story!')

        Raises:
            ValueError: In case of invalid arguments.
            RPCError: In case of Telegram RPCError.

        """
        if business_connection_id:
            business_connection = self.business_user_connection_cache[business_connection_id]
            if not business_connection:
                business_connection = await self.get_business_connection(business_connection_id)

            return await self.post_story(
                chat_id=business_connection.user_chat_id,
                content=content,
                active_period=active_period,
                caption=caption,
                parse_mode=parse_mode,
                caption_entities=caption_entities,
                areas=areas,
                post_to_chat_page=post_to_chat_page,
                protect_content=protect_content,
                privacy_settings=privacy_settings,
                from_story_chat_id=from_story_chat_id,
                from_story_id=from_story_id,
                progress=progress,
                progress_args=progress_args
            )

        message, entities = (await utils.parse_text_entities(self, caption, parse_mode, caption_entities)).values()

        media = None
        thumb = None
        mime_type = None
        if isinstance(content, types.InputStoryContentPhoto):
            media = content.photo
            thumb = content.thumbnail
            mime_type = "image/jpg"
        elif isinstance(content, types.InputStoryContentVideo):
            media = content.video
            thumb = content.thumbnail
            mime_type = "video/mp4"
        else:
            raise ValueError("invalid content")

        try:
            if isinstance(media, str):
                if os.path.isfile(media):
                    file = await self.save_file(media, progress=progress, progress_args=progress_args)
                    thumb = await self.save_file(thumb) if thumb else None
                    if isinstance(content, types.InputStoryContentVideo):
                        media = raw.types.InputMediaUploadedDocument(
                            mime_type=mime_type,
                            file=file,
                            thumb=thumb,
                            attributes=[
                                raw.types.DocumentAttributeVideo(
                                    supports_streaming=content.supports_streaming or None,
                                    duration=content.duration,
                                    w=content.width,
                                    h=content.height,
                                ),
                                raw.types.DocumentAttributeFilename(file_name=content.file_name or os.path.basename(media))
                            ]
                        )
                    else:
                        media = raw.types.InputMediaUploadedPhoto(
                            file=file,
                        )
                else:
                    media = utils.get_input_media_from_file_id(media)
            else:
                file = await self.save_file(media, progress=progress, progress_args=progress_args)
                thumb = await self.save_file(thumb) if thumb else None
                if isinstance(content, types.InputStoryContentVideo):
                    media = raw.types.InputMediaUploadedDocument(
                        mime_type=mime_type,
                        file=file,
                        thumb=thumb,
                        attributes=[
                            raw.types.DocumentAttributeVideo(
                                supports_streaming=content.supports_streaming or None,
                                duration=content.duration,
                                w=content.width,
                                h=content.height,
                            ),
                            raw.types.DocumentAttributeFilename(file_name=content.file_name or media.name)
                        ]
                    )
                else:
                    media = raw.types.InputMediaUploadedPhoto(
                        file=file,
                    )

            privacy_rules = []
            if privacy_settings:
                privacy_rules += (await privacy_settings.write(self))
            else:
                privacy_rules += (await (types.StoryPrivacySettingsEveryone()).write(self))

            while True:
                try:
                    r = await self.invoke(
                        raw.functions.stories.SendStory( 
                            peer=await self.resolve_peer(chat_id),
                            media=media,
                            privacy_rules=privacy_rules,
                            random_id=self.rnd_id(),
                            pinned=post_to_chat_page,
                            noforwards=protect_content,
                            media_areas=[await area.write(self) for area in (areas or [])] or None,
                            caption=message,
                            entities=entities,
                            period=active_period,
                            # fwd_modified=True if from_story_id else None,
                            fwd_from_id=await self.resolve_peer(from_story_chat_id) if from_story_chat_id else None,
                            fwd_from_story=from_story_id,
                        )
                    )
                except FilePartMissing as e:
                    await self.save_file(media, file_id=file.id, file_part=e.value)
                else:
                    for i in r.updates:
                        if isinstance(i, raw.types.UpdateStory):
                            return await types.Story._parse(
                                self,
                                {i.id: i for i in r.users},
                                {i.id: i for i in r.chats},
                                None, None,
                                i,
                                i.story,
                                i.peer
                            )
        except StopTransmission:
            return None



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import io
from typing import Optional, Union

from .input_story_content import InputStoryContent
from ... import enums


class InputStoryContentVideo(InputStoryContent):
    """Describes a video to post as a story.

    It is intended to be used with :obj:`~pyrogram.Client.send_story` or :obj:`~pyrogram.Client.post_story`.

    Parameters:
        video (``str`` | :obj:`io.BytesIO`):
            File to send.
            Pass a file_id as string to send a video that exists on the Telegram servers or
            pass a file path as string to upload a new video that exists on your local machine or
            pass a binary file-like object with its attribute “.name” set for in-memory uploads or
            pass an HTTP URL as a string for Telegram to get a video from the Internet.

        duration (``int``, *optional*):
            Precise duration of the video in seconds; 0-60.

        cover_frame_timestamp (``int``, *optional*):
            Timestamp in seconds of the frame that will be used as the static cover for the story. Defaults to 0.0.

        is_animation (``bool``, *optional*):
            Pass True if the video has no sound.

        width (``int``, *optional*):
            Video width.

        height (``int``, *optional*):
            Video height.

        thumbnail (``str`` | :obj:`io.BytesIO`):
            Thumbnail of the video sent.
            The thumbnail should be in JPEG format and less than 200 KB in size.
            A thumbnail's width and height should not exceed 320 pixels.
            Thumbnails can't be reused and can be only uploaded as a new file.

        supports_streaming (``bool``, *optional*):
            Pass True, if the uploaded video is suitable for streaming.

        file_name (``str``, *optional*):
            File name of the video sent.
            Defaults to file's path basename.

    """

    def __init__(
        self,
        video: Union[str, "io.BytesIO"],
        duration: int = 0,
        cover_frame_timestamp: int = 0,
        is_animation: bool = None,
        width: int = 0,
        height: int = 0,
        thumbnail: Union[str, "io.BytesIO"] = None,
        supports_streaming: bool = True,
        file_name: str = None,
    ):
        super().__init__()

        self.video = video
        self.duration = duration
        self.cover_frame_timestamp = cover_frame_timestamp
        self.is_animation = is_animation
        self.width = width
        self.height = height
        self.thumbnail = thumbnail
        self.supports_streaming = supports_streaming
        self.file_name = file_name



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.


from ..object import Object


class LocationAddress(Object):
    """This object describes the physical address of a location.

    Parameters:
        country_code (``str``):
            The two-letter ISO 3166-1 alpha-2 country code of the country where the location is located.

        state (``str``, *optional*):
            State of the location.

        city (``str``, *optional*):
            City of the location.
        
        street (``str``, *optional*):
            Street address of the location.

    """

    def __init__(
        self,
        country_code: str = None,
        state: str = None,
        city: str = None,
        street: str = None,
    ):
        super().__init__()

        self.country_code = country_code
        self.state = state
        self.city = city
        self.street = street



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.


from typing import Optional, Union

import pyrogram
from pyrogram import raw, types, enums, utils


class SendGift:
    async def send_gift(
        self: "pyrogram.Client",
        *,
        user_id: Union[int, str] = None,
        chat_id: Union[int, str] = None,
        gift_id: int = None,
        pay_for_upgrade: Optional[bool] = None,
        text: Optional[str] = None,
        text_parse_mode: Optional["enums.ParseMode"] = None,
        text_entities: Optional[list["types.MessageEntity"]] = None,
        is_private: Optional[bool] = None,
    ) -> bool:
        """Sends a gift to the given user or channel chat. The gift can't be converted to Telegram Stars by the receiver.

        .. include:: /_includes/usable-by/users-bots.rst

        You must use exactly one of ``user_id`` OR ``chat_id``.

        Parameters:
            user_id (``int`` | ``str``):
                Required if ``chat_id`` is not specified.
                Unique identifier (int) or username (str) of the target user that will receive the gift.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".
                For a contact that exists in your Telegram address book you can use his phone number (str).

            chat_id (``int`` | ``str``):
                Required if ``user_id`` is not specified.
                Unique identifier (int) or username (str) for the chat or username of the channel that will receive the gift.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".
                For a contact that exists in your Telegram address book you can use his phone number (str).

            gift_id (``int``):
                Identifier of the gift.

            pay_for_upgrade (``bool``, *optional*):
                Pass True to pay for the gift upgrade from the sender's balance, thereby making the upgrade free for the receiver.

            text (``str``, *optional*):
                Text that will be shown along with the gift. 0-``gift_text_length_max`` characters.

            text_parse_mode (:obj:`~pyrogram.enums.ParseMode`, *optional*):
                By default, texts are parsed using both Markdown and HTML styles.
                You can combine both syntaxes together.

            text_entities (List of :obj:`~pyrogram.types.MessageEntity`, *optional*):
                List of special entities that appear in message text, which can be specified instead of *text_parse_mode*.
                Only Bold, Italic, Underline, Strikethrough, Spoiler, and CustomEmoji entities are allowed.

            is_private (``bool``, *optional*):
                For users only: Pass True to show the current user as sender and gift text only to the gift receiver; otherwise, everyone will be able to see them.

        Returns:
            ``bool``: On success, True is returned.

        Raises:
            RPCError: In case of a Telegram RPC error.

        Example:
            .. code-block:: python

                # Send gift
                app.send_gift(user_id=user_id, gift_id=123)

        """
        peer = None
        if user_id:
            peer = await self.resolve_peer(user_id)
        elif chat_id:
            peer = await self.resolve_peer(chat_id)
        # TODO
        if not peer:
            raise ValueError("You must use exactly one of user_id OR chat_id")
        text, entities = (await utils.parse_text_entities(self, text, text_parse_mode, text_entities)).values()

        invoice = raw.types.InputInvoiceStarGift(
            user_id=peer,
            gift_id=gift_id,
            hide_name=is_private,
            include_upgrade=pay_for_upgrade,
            message=raw.types.TextWithEntities(
                text=text, entities=entities or []
            ) if text else None
        )

        form = await self.invoke(
            raw.functions.payments.GetPaymentForm(
                invoice=invoice
            )
        )

        await self.invoke(
            raw.functions.payments.SendStarsForm(
                form_id=form.form_id,
                invoice=invoice
            )
        )

        return True



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from typing import Union, Iterable

import pyrogram
from pyrogram import raw, types, utils


class DeleteStories:
    async def delete_stories(
        self: "pyrogram.Client",
        chat_id: Union[int, str],
        story_ids: Union[int, Iterable[int]],
    ) -> list[int]:
        """Deletes a previously sent story.

        .. include:: /_includes/usable-by/users.rst

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier (int) or username (str) of the target chat.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".

            story_ids (``int`` | Iterable of ``int``, *optional*):
                Unique identifier (int) or list of unique identifiers (list of int) for the target stories.

        Returns:
            List of ``int``: List of deleted stories IDs.

        Example:
            .. code-block:: python

                # Delete a single story
                app.delete_stories(chat_id, 1)

                # Delete multiple stories
                app.delete_stories(chat_id, [1, 2])

        """
        is_iterable = utils.is_list_like(story_ids)
        ids = list(story_ids) if is_iterable else [story_ids]
        r = await self.invoke(
            raw.functions.stories.DeleteStories(
                peer=await self.resolve_peer(chat_id),
                id=ids
            )
        )
        return types.List(r)


    async def delete_business_story(
        self: "pyrogram.Client",
        business_connection_id: str,
        story_id: int,
    ) -> list[int]:
        """Deletes a story previously posted by the bot on behalf of a managed business account.
        
        Requires the can_manage_stories business bot right.

        .. include:: /_includes/usable-by/bots.rst

        Parameters:
            business_connection_id (``str``):
                Unique identifier of the business connection.
            
            story_id (``int``):
                Unique identifier of the story to delete.
        
        Returns:
            List of ``int``: List of deleted stories IDs.

        """
        if not business_connection_id:
            raise ValueError("business_connection_id is required")

        business_connection = self.business_user_connection_cache[business_connection_id]
        if business_connection is None:
            business_connection = await self.get_business_connection(business_connection_id)
        return await self.delete_stories(
            chat_id=business_connection.user_chat_id,
            story_ids=story_id
        )



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import pyrogram
from pyrogram import raw


class CountPublicMessagesByTag:
    async def count_public_messages_by_tag(
        self: "pyrogram.Client",
        tag: str = "",
    ) -> int:
        """Get the count of messages with the provided hashtag or cashtag.

        If you want to get the actual messages, see :meth:`~pyrogram.Client.search_public_messages_by_tag`.

        .. include:: /_includes/usable-by/users.rst

        Parameters:
            hashtag (``str``, *optional*):
                Hashtag or cashtag to search for.

        Returns:
            ``int``: On success, the messages count is returned.

        """
        r = await self.invoke(
            raw.functions.channels.SearchPosts(
                hashtag=tag,
                offset_rate=0,
                offset_peer=raw.types.InputPeerEmpty(),
                offset_id=0,
                limit=1
            )
        )

        if hasattr(r, "count"):
            return r.count
        else:
            return len(r.messages)



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
from typing import Optional, Union

import pyrogram
from pyrogram import raw, types, utils
from .message import Str
from ..object import Object


class ReceivedGift(Object):
    """Represents a gift received by a user or a chat.

    Parameters:
        sender_user (:obj:`~pyrogram.types.User`, *optional*):
            Identifier of the user that sent the gift; None if unknown.

        text (``str``, *optional*):
            Message added to the gift.
        
        entities (List of :obj:`~pyrogram.types.MessageEntity`, *optional*):
            For text messages, special entities like usernames, URLs, bot commands, etc. that appear in the text.

        date (:py:obj:`~datetime.datetime`, *optional*):
            Date when the gift was sent.

        is_private (``bool``, *optional*):
            True, if the sender and gift text are shown only to the gift receiver; otherwise, everyone are able to see them.

        is_saved (``bool``, *optional*):
            True, if the gift is displayed on the user's profile page; may be False only for the receiver of the gift.

        is_pinned (``bool``, *optional*):
            True, if the gift is pinned to the top of the chat's profile page.

        gift (:obj:`~pyrogram.types.Gift` | :obj:`~pyrogram.types.UpgradedGift`, *optional*):
            Information about the gift.
        
        message_id (``int``, *optional*):
            Identifier of the message with the gift in the chat with the sender of the gift; can be None or an identifier of a deleted message; only for the gift receiver.

        sell_star_count (``int``, *optional*):
            Number of Telegram Stars that can be claimed by the receiver instead of the regular gift; 0 if the gift can't be sold by the current user.

        was_converted (``bool``, *optional*):
            True, if the gift was converted to Telegram Stars; only for the receiver of the gift.

        can_be_upgraded (``bool``, *optional*):
            True, if the gift is a regular gift that can be upgraded to a unique gift; only for the receiver of the gift.

        was_refunded (``bool``, *optional*):
            True, if the gift was refunded and isn't available anymore.

        prepaid_upgrade_star_count (``int``, *optional*):
            Number of Telegram Stars that were paid by the sender for the ability to upgrade the gift.

        can_be_transferred (``bool``, *optional*):
            True, if the gift is an upgraded gift that can be transferred to another user; only for the receiver of the gift.

        transfer_star_count (``int``, *optional*):
            Number of Telegram Stars that must be paid to transfer the upgraded gift; only for the receiver of the gift.

        export_date (:py:obj:`~datetime.datetime`, *optional*):
            Point in time (Unix timestamp) when the upgraded gift can be transferred to TON blockchain as an NFT; None if NFT export isn't possible; only for the receiver of the gift.

    """

    def __init__(
        self,
        *,
        client: "pyrogram.Client" = None,
        sender_user: Optional["types.User"] = None,
        text: Optional[Str] = None,
        entities: list["types.MessageEntity"] = None,
        date: datetime,
        is_private: Optional[bool] = None,
        is_saved: Optional[bool] = None,
        is_pinned: Optional[bool] = None,
        gift: Optional[Union["types.Gift", "types.UpgradedGift"]] = None,
        message_id: Optional[int] = None,
        sell_star_count: Optional[int] = None,
        was_converted: Optional[bool] = None,
        can_be_upgraded: Optional[bool] = None,
        was_refunded: Optional[bool] = None,
        prepaid_upgrade_star_count: Optional[int] = None,
        can_be_transferred: Optional[bool] = None,
        transfer_star_count: Optional[int] = None,
        export_date: datetime = None,
    ):
        super().__init__(client)

        self.sender_user = sender_user
        self.text = text
        self.entities = entities
        self.date = date
        self.is_private = is_private
        self.is_saved = is_saved
        self.is_pinned = is_pinned
        self.gift = gift
        self.message_id = message_id
        self.sell_star_count = sell_star_count
        self.was_converted = was_converted
        self.can_be_upgraded = can_be_upgraded
        self.was_refunded = was_refunded
        self.prepaid_upgrade_star_count = prepaid_upgrade_star_count
        self.can_be_transferred = can_be_transferred
        self.transfer_star_count = transfer_star_count
        self.export_date = export_date


    @staticmethod
    async def _parse(
        client,
        saved_star_gift: "raw.types.SavedStarGift",
        users: dict,
        chats: dict
    ) -> "ReceivedGift":
        text, entities = None, None
        if getattr(saved_star_gift, "message", None):
            text = saved_star_gift.message.text or None
            entities = [types.MessageEntity._parse(client, entity, users) for entity in saved_star_gift.message.entities]
            entities = types.List(filter(lambda x: x is not None, entities))
        sender_user = utils.get_raw_peer_id(saved_star_gift.from_id)
        return ReceivedGift(
            sender_user=types.User._parse(
                client,
                users.get(sender_user)
            ) if sender_user else None,
            text=Str(text).init(entities) if text else None,
            entities=entities,
            date=utils.timestamp_to_datetime(saved_star_gift.date),
            is_private=getattr(saved_star_gift, "name_hidden", None),
            is_saved=not saved_star_gift.unsaved if getattr(saved_star_gift, "unsaved", False) else None,
            is_pinned=saved_star_gift.pinned_to_top,
            gift=await types.Gift._parse(
                client,
                saved_star_gift.gift,
                users
            ),
            message_id=getattr(saved_star_gift, "msg_id", None),
            sell_star_count=getattr(saved_star_gift, "convert_stars", 0),
            was_converted=bool(getattr(saved_star_gift, "saved_id", None)),
            can_be_upgraded=getattr(saved_star_gift, "can_upgrade", None),
            was_refunded=getattr(saved_star_gift, "refunded", None),
            prepaid_upgrade_star_count=getattr(saved_star_gift, "upgrade_stars", None),
            can_be_transferred=bool(getattr(saved_star_gift, "transfer_stars", None)),
            transfer_star_count=getattr(saved_star_gift, "transfer_stars", None),
            export_date=utils.timestamp_to_datetime(saved_star_gift.can_export_at),
            client=client
        )

    @staticmethod
    async def _parse_action(
        client,
        message: "raw.base.Message",
        users: dict,
        chats: dict
    ) -> "ReceivedGift":
        action = message.action

        text, entities = None, None
        if getattr(action, "message", None):
            text = action.message.text or None
            entities = [types.MessageEntity._parse(client, entity, users) for entity in action.message.entities]
            entities = types.List(filter(lambda x: x is not None, entities))
        # TODO 
        if isinstance(action, raw.types.MessageActionStarGift):
            return ReceivedGift(
                gift=await types.Gift._parse(
                    client,
                    action.gift,
                    users
                ),
                date=utils.timestamp_to_datetime(message.date),
                is_private=getattr(action, "name_hidden", None),
                is_saved=getattr(action, "saved", None),
                sender_user=types.User._parse(client, users.get(utils.get_raw_peer_id(message.peer_id))),
                message_id=message.id,
                text=Str(text).init(entities) if text else None,
                entities=entities,
                sell_star_count=getattr(action, "convert_stars", None),
                was_converted=getattr(action, "converted", None),
                can_be_upgraded=getattr(action, "can_upgrade", None),
                was_refunded=getattr(action, "refunded", None),
                prepaid_upgrade_star_count=getattr(action, "upgrade_stars", None),
                client=client
            )

        if isinstance(action, raw.types.MessageActionStarGiftUnique):
            return ReceivedGift(
                gift=await types.Gift._parse(
                    client,
                    action.gift,
                    users
                ),
                date=utils.timestamp_to_datetime(message.date),
                sender_user=types.User._parse(client, users.get(utils.get_raw_peer_id(message.peer_id))),
                message_id=message.id,
                text=Str(text).init(entities) if text else None,
                entities=entities,
                can_be_transferred=getattr(action, "transferred", None),
                was_refunded=getattr(action, "refunded", None),
                prepaid_upgrade_star_count=getattr(action, "upgrade_stars", None),
                export_date=utils.timestamp_to_datetime(getattr(action, "can_export_at", None)),
                transfer_star_count=getattr(action, "transfer_stars", None),
                client=client
            )

    async def toggle(self, is_saved: bool) -> bool:
        """Bound method *toggle* of :obj:`~pyrogram.types.ReceivedGift`.

        Use as a shortcut for:

        .. code-block:: python

            await client.toggle_gift_is_saved(
                message_id=message_id
            )

        Parameters:
            is_saved (``bool``):
                Pass True to display the gift on the user's profile page; pass False to remove it from the profile page.

        Example:
            .. code-block:: python

                await received_gift.toggle(is_saved=False)

        Returns:
            ``bool``: On success, True is returned.

        """
        return await self._client.toggle_gift_is_saved(
            message_id=self.message_id,
            is_saved=is_saved
        )



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import logging
import io
import os
import re
from datetime import datetime
from typing import Union, Optional, Callable

import pyrogram
from pyrogram import raw, enums, types, utils
from pyrogram.errors import FilePartMissing
from pyrogram.file_id import FileType
from .inline_session import get_session

log = logging.getLogger(__name__)


class SendPhoto:
    async def send_photo(
        self: "pyrogram.Client",
        chat_id: Union[int, str],
        photo: Union[str, "io.BytesIO"],
        caption: str = "",
        parse_mode: Optional["enums.ParseMode"] = None,
        caption_entities: list["types.MessageEntity"] = None,
        show_caption_above_media: bool = None,
        has_spoiler: bool = None,
        ttl_seconds: int = None,
        disable_notification: bool = None,
        reply_parameters: "types.ReplyParameters" = None,
        message_thread_id: int = None,
        business_connection_id: str = None,
        send_as: Union[int, str] = None,
        schedule_date: datetime = None,
        protect_content: bool = None,
        allow_paid_broadcast: bool = None,
        paid_message_star_count: int = None,
        view_once: bool = None,
        message_effect_id: int = None,
        reply_markup: Union[
            "types.InlineKeyboardMarkup",
            "types.ReplyKeyboardMarkup",
            "types.ReplyKeyboardRemove",
            "types.ForceReply"
        ] = None,
        reply_to_message_id: int = None,
        progress: Callable = None,
        progress_args: tuple = ()
    ) -> Optional["types.Message"]:
        """Send photos.

        .. include:: /_includes/usable-by/users-bots.rst

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier (int) or username (str) of the target chat.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".
                For a contact that exists in your Telegram address book you can use his phone number (str).

            photo (``str`` | :obj:`io.BytesIO`):
                Photo to send.
                Pass a file_id as string to send a photo that exists on the Telegram servers,
                pass an HTTP URL as a string for Telegram to get a photo from the Internet,
                pass a file path as string to upload a new photo that exists on your local machine, or
                pass a binary file-like object with its attribute ".name" set for in-memory uploads.
                The photo must be at most 10 MB in size.
                The photo's width and height must not exceed 10000 in total.
                The photo's width and height ratio must be at most 20.

            caption (``str``, *optional*):
                Photo caption, 0-1024 characters.

            parse_mode (:obj:`~pyrogram.enums.ParseMode`, *optional*):
                By default, texts are parsed using both Markdown and HTML styles.
                You can combine both syntaxes together.

            caption_entities (List of :obj:`~pyrogram.types.MessageEntity`):
                List of special entities that appear in the caption, which can be specified instead of *parse_mode*.

            show_caption_above_media (``bool``, *optional*):
                Pass True, if the caption must be shown above the message media.

            has_spoiler (``bool``, *optional*):
                Pass True if the photo needs to be covered with a spoiler animation.

            ttl_seconds (``int``, *optional*):
                The message will be self-destructed in the specified time after its content was opened.
                The message's self-destruct time, in seconds; must be between 0 and 60 in private chats.

            disable_notification (``bool``, *optional*):
                Sends the message silently.
                Users will receive a notification with no sound.

            reply_parameters (:obj:`~pyrogram.types.ReplyParameters`, *optional*):
                Description of the message to reply to

            message_thread_id (``int``, *optional*):
                If the message is in a thread, ID of the original message.

            business_connection_id (``str``, *optional*):
                Unique identifier of the business connection on behalf of which the message will be sent.

            send_as (``int`` | ``str``):
                Unique identifier (int) or username (str) of the chat or channel to send the message as.
                You can use this to send the message on behalf of a chat or channel where you have appropriate permissions.
                Use the :meth:`~pyrogram.Client.get_send_as_chats` to return the list of message sender identifiers, which can be used to send messages in the chat, 
                This setting applies to the current message and will remain effective for future messages unless explicitly changed.
                To set this behavior permanently for all messages, use :meth:`~pyrogram.Client.set_send_as_chat`.

            schedule_date (:py:obj:`~datetime.datetime`, *optional*):
                Date when the message will be automatically sent.

            protect_content (``bool``, *optional*):
                Pass True if the content of the message must be protected from forwarding and saving; for bots only.

            allow_paid_broadcast (``bool``, *optional*):
                Pass True to allow the message to ignore regular broadcast limits for a small fee; for bots only

            paid_message_star_count (``int``, *optional*):
                The number of Telegram Stars the user agreed to pay to send the messages.

            view_once (``bool``, *optional*):
                Pass True if the message should be opened only once and should be self-destructed once closed; private chats only.

            message_effect_id (``int`` ``64-bit``, *optional*):
                Unique identifier of the message effect to be added to the message; for private chats only.

            reply_markup (:obj:`~pyrogram.types.InlineKeyboardMarkup` | :obj:`~pyrogram.types.ReplyKeyboardMarkup` | :obj:`~pyrogram.types.ReplyKeyboardRemove` | :obj:`~pyrogram.types.ForceReply`, *optional*):
                Additional interface options. An object for an inline keyboard, custom reply keyboard,
                instructions to remove reply keyboard or to force a reply from the user.

            progress (``Callable``, *optional*):
                Pass a callback function to view the file transmission progress.
                The function must take *(current, total)* as positional arguments (look at Other Parameters below for a
                detailed description) and will be called back each time a new file chunk has been successfully
                transmitted.

            progress_args (``tuple``, *optional*):
                Extra custom arguments for the progress callback function.
                You can pass anything you need to be available in the progress callback scope; for example, a Message
                object or a Client instance in order to edit the message with the updated progress status.

        Other Parameters:
            current (``int``):
                The amount of bytes transmitted so far.

            total (``int``):
                The total size of the file.

            *args (``tuple``, *optional*):
                Extra custom arguments as defined in the ``progress_args`` parameter.
                You can either keep ``*args`` or add every single extra argument in your function signature.

        Returns:
            :obj:`~pyrogram.types.Message` | ``None``: On success, the sent photo message is returned, otherwise, in
            case the upload is deliberately stopped with :meth:`~pyrogram.Client.stop_transmission`, None is returned.

        Example:
            .. code-block:: python

                # Send photo by uploading from local file
                await app.send_photo("me", "photo.jpg")

                # Send photo by uploading from URL
                await app.send_photo("me", "https://example.com/example.jpg")

                # Add caption to a photo
                await app.send_photo("me", "photo.jpg", caption="Caption")

                # Send self-destructing photo
                await app.send_photo("me", "photo.jpg", ttl_seconds=10)
        """

        if reply_to_message_id and reply_parameters:
            raise ValueError(
                "Parameters `reply_to_message_id` and `reply_parameters` are mutually "
                "exclusive."
            )
        
        if reply_to_message_id is not None:
            log.warning(
                "This property is deprecated. "
                "Please use reply_parameters instead"
            )
            reply_parameters = types.ReplyParameters(message_id=reply_to_message_id)

        file = None
        ttl_seconds = 0x7FFFFFFF if view_once else ttl_seconds

        try:
            if isinstance(photo, str):
                if os.path.isfile(photo):
                    file = await self.save_file(photo, progress=progress, progress_args=progress_args)
                    media = raw.types.InputMediaUploadedPhoto(
                        file=file,
                        ttl_seconds=ttl_seconds,
                        spoiler=has_spoiler,
                    )
                elif re.match("^https?://", photo):
                    media = raw.types.InputMediaPhotoExternal(
                        url=photo,
                        ttl_seconds=ttl_seconds,
                        spoiler=has_spoiler
                    )
                else:
                    media = utils.get_input_media_from_file_id(
                        photo,
                        FileType.PHOTO,
                        ttl_seconds=ttl_seconds,
                        has_spoiler=has_spoiler
                    )
            else:
                file = await self.save_file(photo, progress=progress, progress_args=progress_args)
                media = raw.types.InputMediaUploadedPhoto(
                    file=file,
                    ttl_seconds=ttl_seconds,
                    spoiler=has_spoiler
                )

            reply_to = await utils._get_reply_message_parameters(
                self,
                message_thread_id,
                reply_parameters
            )
            rpc = raw.functions.messages.SendMedia(
                peer=await self.resolve_peer(chat_id),
                media=media,
                silent=disable_notification or None,
                reply_to=reply_to,
                random_id=self.rnd_id(),
                send_as=await self.resolve_peer(send_as) if send_as else None,
                schedule_date=utils.datetime_to_timestamp(schedule_date),
                noforwards=protect_content,
                allow_paid_floodskip=allow_paid_broadcast,
                allow_paid_stars=paid_message_star_count,
                reply_markup=await reply_markup.write(self) if reply_markup else None,
                effect=message_effect_id,
                invert_media=show_caption_above_media,
                **await utils.parse_text_entities(self, caption, parse_mode, caption_entities)
            )
            session = None
            business_connection = None
            if business_connection_id:
                business_connection = self.business_user_connection_cache[business_connection_id]
                if business_connection is None:
                    business_connection = await self.get_business_connection(business_connection_id)
                session = await get_session(
                    self,
                    business_connection._raw.connection.dc_id
                )

            while True:
                try:
                    if business_connection_id:
                        r = await session.invoke(
                            raw.functions.InvokeWithBusinessConnection(
                                query=rpc,
                                connection_id=business_connection_id
                            )
                        )
                        # await session.stop()
                    else:
                        r = await self.invoke(rpc)
                except FilePartMissing as e:
                    await self.save_file(photo, file_id=file.id, file_part=e.value)
                else:
                    for i in r.updates:
                        if isinstance(
                            i,
                            (
                                raw.types.UpdateNewMessage,
                                raw.types.UpdateNewChannelMessage,
                                raw.types.UpdateNewScheduledMessage
                            )
                        ):
                            return await types.Message._parse(
                                self, i.message,
                                {i.id: i for i in r.users},
                                {i.id: i for i in r.chats},
                                is_scheduled=isinstance(i, raw.types.UpdateNewScheduledMessage),
                                replies=self.fetch_replies
                            )
                        elif isinstance(
                            i,
                            (
                                raw.types.UpdateBotNewBusinessMessage
                            )
                        ):
                            return await types.Message._parse(
                                self,
                                i.message,
                                {i.id: i for i in r.users},
                                {i.id: i for i in r.chats},
                                business_connection_id=getattr(i, "connection_id", business_connection_id),
                                raw_reply_to_message=i.reply_to_message,
                                replies=0
                            )
        except pyrogram.StopTransmission:
            return None



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.


import pyrogram
from pyrogram import raw, types, utils

from ..object import Object


class StoryPrivacySettings(Object):
    """This object describes privacy settings of a story.

    Currently, it can be one of:

    - :obj:`~pyrogram.types.StoryPrivacySettingsEveryone`
    - :obj:`~pyrogram.types.StoryPrivacySettingsContacts`
    - :obj:`~pyrogram.types.StoryPrivacySettingsCloseFriends`
    - :obj:`~pyrogram.types.StoryPrivacySettingsSelectedUsers`

    """

    def __init__(self):
        super().__init__()

    @staticmethod
    def _parse(
        client: "pyrogram.Client",
        privacy_rules: list["raw.base.PrivacyRule"],
    ) -> "StoryPrivacySettings":
        ko = []
        for priv in privacy_rules:
            # privacyValueDisallowContacts#f888fa1a = PrivacyRule;
            # privacyValueDisallowAll#8b73e763 = PrivacyRule;
            # privacyValueDisallowUsers#e4621141 users:Vector<long> = PrivacyRule;
            # privacyValueDisallowChatParticipants#41c87565 chats:Vector<long> = PrivacyRule;
            # privacyValueAllowPremium#ece9814b = PrivacyRule;
            # privacyValueAllowBots#21461b5d = PrivacyRule;
            # privacyValueDisallowBots#f6a5f82f = PrivacyRule;
            if isinstance(priv, raw.types.PrivacyValueAllowAll):
                ko.append(types.StoryPrivacySettingsEveryone())
            if isinstance(priv, raw.types.PrivacyValueDisallowUsers):
                if types.StoryPrivacySettingsEveryone in ko:
                    ko.append(types.StoryPrivacySettingsEveryone(except_user_ids=priv.users))
                if types.StoryPrivacySettingsContacts in ko:
                    ko.append(types.StoryPrivacySettingsContacts(except_user_ids=priv.users))
            if isinstance(priv, raw.types.PrivacyValueAllowContacts):
                ko.append(types.StoryPrivacySettingsContacts())
            if isinstance(priv, raw.types.PrivacyValueAllowCloseFriends):
                ko.append(types.StoryPrivacySettingsCloseFriends())
            if isinstance(priv, raw.types.PrivacyValueAllowUsers):
                ko.append(types.StoryPrivacySettingsSelectedUsers(user_ids=priv.users))
            if isinstance(priv, raw.types.PrivacyValueAllowChatParticipants):
                ko.append(types.StoryPrivacySettingsSelectedUsers(user_ids=[utils.get_channel_id(chat_id) for chat_id in priv.chats]))
        return ko[-1] if len(ko) > 0 else None



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import logging
from asyncio import sleep
from datetime import datetime
from typing import AsyncGenerator, Optional, Union 

import pyrogram
from pyrogram import raw, types, utils

log = logging.getLogger(__name__)


async def get_chunk(
    *,
    client: "pyrogram.Client",
    chat_id: Union[int, str],
    limit: int = 0,
    offset: int = 0,
    min_id: int = 0,
    max_id: int = 0,
    offset_id: int = 0,
    from_date: datetime = utils.zero_datetime(),
    reverse: bool = False,
    is_scheduled: bool = False
) -> list[types.Message]:
    if is_scheduled:
        r = await client.invoke(
            raw.functions.messages.GetScheduledHistory(
                peer=await client.resolve_peer(chat_id),
                hash=0
            ),
            sleep_threshold=60
        )
        messages = await utils.parse_messages(
            client,
            r,
            is_scheduled=True,
            replies=0
        )
        if reverse:
            messages.reverse()
        return messages
    else:
        if (min_id or max_id) and not offset_id:
            if max_id:
                offset_id = max_id + 1
            elif min_id:
                offset_id = 0
        if min_id and max_id and not offset_id:
            offset_id = max_id + 1
        history: raw.base.messages.Messages = await client.invoke(
            raw.functions.messages.GetHistory(
                peer=await client.resolve_peer(chat_id),
                offset_id=offset_id,
                offset_date=utils.datetime_to_timestamp(from_date),
                add_offset=offset,
                limit=limit,
                max_id=max_id,
                min_id=min_id,
                hash=0
            ),
            sleep_threshold=60
        )
        messages = await utils.parse_messages(
            client,
            history,
            is_scheduled=False,
            replies=0
        )
        if reverse:
            messages.reverse()
        return messages


class GetChatHistory:
    async def get_chat_history(
        self: "pyrogram.Client",
        chat_id: Union[int, str],
        limit: int = 0,
        *,
        offset: int = 0,
        offset_id: int = None,
        min_id: int = 0,
        max_id: int = 0,
        offset_date: datetime = utils.zero_datetime(),
        reverse: bool = False,
        is_scheduled: bool = False
    ) -> Optional[AsyncGenerator["types.Message", None]]:
        """Get messages from a chat history.

        The messages are returned in reverse chronological order by default.

        .. include:: /_includes/usable-by/users.rst

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier (int) or username (str) of the target chat.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".
                For a contact that exists in your Telegram address book you can use his phone number (str).

            limit (``int``, *optional*):
                Limits the number of messages to be retrieved.
                By default, no limit is applied and all messages are returned.

            offset (``int``, *optional*):
                Sequential number of the first message to be returned.
                Negative values are also accepted and become useful in case you set offset_id or offset_date.

            offset_id (``int``, *optional*):
                Identifier of the first message to be returned.
                This parameter is deprecated and should not be used.
                Use ``min_id`` / ``max_id`` instead for proper filtering.

            min_id (``int``, *optional*):
                If a positive value was transferred, the method will return only messages with IDs more than min_id.
                Defaults to 0.

            max_id (``int``, *optional*):
                If a positive value was transferred, the method will return only messages with IDs less than max_id.
                Defaults to 0.

            offset_date (:py:obj:`~datetime.datetime`, *optional*):
                Pass a date as offset to retrieve only older messages starting from that date.

            reverse (``bool``, *optional*):
                Pass True to retrieve the messages in reversed order (from older to most recent). Defaults to False.

            is_scheduled (``bool``, *optional*):
                Whether to get scheduled messages. Defaults to False.

        Returns:
            ``Generator``: A generator yielding :obj:`~pyrogram.types.Message` objects.

        Example:
            .. code-block:: python

                async for message in app.get_chat_history(chat_id):
                    print(message.text)
        """
        if offset_id is not None:
            log.warning(
                "`offset_id` is deprecated and will be removed in future updates. Use `min_id` or `max_id` instead."
            )

        current: int = 0
        total: int = limit or (1 << 31) - 1
        limit: int = min(100, total)

        if reverse:
            offset_id = min_id if min_id else 1
            offset = offset - limit
        else:
            offset_id = max_id if max_id else 0

        while True:
            messages = await get_chunk(
                client=self,
                chat_id=chat_id,
                limit=limit,
                offset=offset,
                offset_id=offset_id,
                min_id=min_id,
                max_id=max_id,
                from_date=offset_date,
                reverse=reverse,
                is_scheduled=is_scheduled
            )

            if not messages:
                return

            offset_id = messages[-1].id + (1 if reverse else 0)

            for message in messages:
                await sleep(0)
                yield message

                current += 1

                if current >= total:
                    return

            if is_scheduled:
                break



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import os
from typing import BinaryIO, Callable, Union

import pyrogram
from pyrogram import StopTransmission, enums, raw, types, utils
from pyrogram.errors import FilePartMissing


class EditStory:
    async def edit_story(
        self: "pyrogram.Client",
        chat_id: Union[int, str],
        story_id: int,
        content: "types.InputStoryContent" = None,
        caption: str = None,
        parse_mode: "enums.ParseMode" = None,
        caption_entities: list["types.MessageEntity"] = None,
        areas: list["types.StoryArea"] = None,
        privacy_settings: "types.StoryPrivacySettings" = None,
        progress: Callable = None,
        progress_args: tuple = (),
    ) -> "types.Story":
        """Changes content, privacy settings and caption of a story.

        .. include:: /_includes/usable-by/users.rst

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier (int) or username (str) of the target chat.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".

            story_id (``int``):
                Unique identifier of the story to edit.

            content (:obj:`~pyrogram.types.InputStoryContent`, *optional*):
                Content of the story.

            caption (``str``, *optional*):
                Caption of the story, 0-2048 characters after entities parsing.
            
            parse_mode (:obj:`~pyrogram.enums.ParseMode`, *optional*):
                By default, texts are parsed using both Markdown and HTML styles.
                You can combine both syntaxes together.

            caption_entities (List of :obj:`~pyrogram.types.MessageEntity`):
                List of special entities that appear in the caption, which can be specified instead of *parse_mode*.

            areas (List of :obj:`~pyrogram.types.StoryArea`, *optional*):
                List of of clickable areas to be shown on the story.

            privacy_settings (:obj:`~pyrogram.types.StoryPrivacySettings`, *optional*):
                The privacy settings for the story; ignored for stories sent to supergroup and channel chats.
                Defaults to :obj:`~pyrogram.types.StoryPrivacySettingsEveryone`.

            progress (``Callable``, *optional*):
                Pass a callback function to view the file transmission progress.
                The function must take *(current, total)* as positional arguments (look at Other Parameters below for a
                detailed description) and will be called back each time a new file chunk has been successfully
                transmitted.

            progress_args (``tuple``, *optional*):
                Extra custom arguments for the progress callback function.
                You can pass anything you need to be available in the progress callback scope; for example, a Message
                object or a Client instance in order to edit the message with the updated progress status.

        Returns:
            :obj:`~pyrogram.types.Story` a single story is returned.

        Example:
            .. code-block:: python

                # Post story to your profile
                await app.edit_story("me", 7, "story.png", caption='My new story!')

                # Post story to channel
                await app.send_story(123456, 3, "story.png", caption='My new story!')

        Raises:
            ValueError: In case of invalid arguments.
            RPCError: In case of Telegram RPCError.

        """

        message, entities = (await utils.parse_text_entities(self, caption, parse_mode, caption_entities)).values()

        media = None
        if content:
            thumb = None
            mime_type = None
            if isinstance(content, types.InputStoryContentPhoto):
                media = content.photo
                thumb = content.thumbnail
                mime_type = "image/jpg"
            elif isinstance(content, types.InputStoryContentVideo):
                media = content.video
                thumb = content.thumbnail
                mime_type = "video/mp4"

            try:
                if isinstance(media, str):
                    if os.path.isfile(media):
                        file = await self.save_file(media, progress=progress, progress_args=progress_args)
                        thumb = await self.save_file(thumb) if thumb else None
                        if isinstance(content, types.InputStoryContentVideo):
                            media = raw.types.InputMediaUploadedDocument(
                                mime_type=mime_type,
                                file=file,
                                thumb=thumb,
                                attributes=[
                                    raw.types.DocumentAttributeVideo(
                                        supports_streaming=content.supports_streaming or None,
                                        duration=content.duration,
                                        w=content.width,
                                        h=content.height,
                                    ),
                                    raw.types.DocumentAttributeFilename(file_name=content.file_name or os.path.basename(media))
                                ]
                            )
                        else:
                            media = raw.types.InputMediaUploadedPhoto(
                                file=file,
                            )
                    else:
                        media = utils.get_input_media_from_file_id(media)
                else:
                    file = await self.save_file(media, progress=progress, progress_args=progress_args)
                    thumb = await self.save_file(thumb) if thumb else None
                    if isinstance(content, types.InputStoryContentVideo):
                        media = raw.types.InputMediaUploadedDocument(
                            mime_type=mime_type,
                            file=file,
                            thumb=thumb,
                            attributes=[
                                raw.types.DocumentAttributeVideo(
                                    supports_streaming=content.supports_streaming or None,
                                    duration=content.duration,
                                    w=content.width,
                                    h=content.height,
                                ),
                                raw.types.DocumentAttributeFilename(file_name=content.file_name or media.name)
                            ]
                        )
                    else:
                        media = raw.types.InputMediaUploadedPhoto(
                            file=file,
                        )
            except StopTransmission:
                return None

        privacy_rules = []
        if privacy_settings:
            privacy_rules += (await privacy_settings.write(self))
        else:
            privacy_rules += (await (types.StoryPrivacySettingsEveryone()).write(self))

        r = await self.invoke(
            raw.functions.stories.EditStory( 
                peer=await self.resolve_peer(chat_id),
                id=story_id,
                media=media,
                media_areas=[await area.write(self) for area in (areas or [])] or None,
                caption=message,
                entities=entities,
                privacy_rules=privacy_rules,
            )
        )

        for i in r.updates:
            if isinstance(i, raw.types.UpdateStory):
                return await types.Story._parse(
                    self,
                    {i.id: i for i in r.users},
                    {i.id: i for i in r.chats},
                    None, None,
                    i,
                    i.story,
                    i.peer
                )


    async def edit_business_story(
        self: "pyrogram.Client",
        business_connection_id: str,
        story_id: int,
        content: "types.InputStoryContent" = None,
        caption: str = None,
        parse_mode: "enums.ParseMode" = None,
        caption_entities: list["types.MessageEntity"] = None,
        areas: list["types.StoryArea"] = None,
        privacy_settings: "types.StoryPrivacySettings" = None,
        progress: Callable = None,
        progress_args: tuple = (),
    ) -> "types.Story":
        """Edits a story previously posted by the bot on behalf of a managed business account. Requires the can_manage_stories business bot right.

        .. include:: /_includes/usable-by/bots.rst

        Parameters:
            business_connection_id (``str``):
                Unique identifier of the business connection.

            story_id (``int``):
                Unique identifier of the story to edit.

            content (:obj:`~pyrogram.types.InputStoryContent`, *optional*):
                Content of the story.

            caption (``str``, *optional*):
                Caption of the story, 0-2048 characters after entities parsing.
            
            parse_mode (:obj:`~pyrogram.enums.ParseMode`, *optional*):
                By default, texts are parsed using both Markdown and HTML styles.
                You can combine both syntaxes together.

            caption_entities (List of :obj:`~pyrogram.types.MessageEntity`):
                List of special entities that appear in the caption, which can be specified instead of *parse_mode*.

            areas (List of :obj:`~pyrogram.types.StoryArea`, *optional*):
                List of of clickable areas to be shown on the story.

            privacy_settings (:obj:`~pyrogram.types.StoryPrivacySettings`, *optional*):
                The privacy settings for the story; ignored for stories sent to supergroup and channel chats.
                Defaults to :obj:`~pyrogram.types.StoryPrivacySettingsEveryone`.

            progress (``Callable``, *optional*):
                Pass a callback function to view the file transmission progress.
                The function must take *(current, total)* as positional arguments (look at Other Parameters below for a
                detailed description) and will be called back each time a new file chunk has been successfully
                transmitted.

            progress_args (``tuple``, *optional*):
                Extra custom arguments for the progress callback function.
                You can pass anything you need to be available in the progress callback scope; for example, a Message
                object or a Client instance in order to edit the message with the updated progress status.

        Returns:
            :obj:`~pyrogram.types.Story` a story is returned.

        Raises:
            ValueError: In case of invalid arguments.
            RPCError: In case of Telegram RPCError.

        """
        if not business_connection_id:
            raise ValueError("business_connection_id is required")

        business_connection = self.business_user_connection_cache[business_connection_id]
        if business_connection is None:
            business_connection = await self.get_business_connection(business_connection_id)
        
        return await self.edit_story(
            chat_id=business_connection.user_chat_id,
            story_id=story_id,
            content=content,
            caption=caption,
            parse_mode=parse_mode,
            caption_entities=caption_entities,
            areas=areas,
            privacy_settings=privacy_settings,
            progress=progress,
            progress_args=progress_args
        )



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from ..object import Object

import pyrogram
from pyrogram import raw, types


class PaidReactionType(Object):
    """This object describes the type of paid message reaction.
    
    It can be one of:

    - :obj:`~pyrogram.types.PaidReactionTypeRegular`
    - :obj:`~pyrogram.types.PaidReactionTypeAnonymous`
    - :obj:`~pyrogram.types.PaidReactionTypeChat`

    """

    def __init__(self):
        super().__init__()
    
    async def write(
        self,
        client: "pyrogram.Client",
    ):
        if isinstance(self, PaidReactionTypeChat):
            return self._raw(
                peer=await client.resolve_peer(self.chat_id)
            )
        else:
            return self._raw()



class PaidReactionTypeRegular(PaidReactionType):
    """A paid reaction on behalf of the current user.

    """
    def __init__(self):
        super().__init__()

        self._raw = raw.types.PaidReactionPrivacyDefault


class PaidReactionTypeAnonymous(PaidReactionType):
    """An anonymous paid reaction.
    
    """
    def __init__(self):
        super().__init__()

        self._raw = raw.types.PaidReactionPrivacyAnonymous


class PaidReactionTypeChat(PaidReactionType):
    """A paid reaction on behalf of an owned chat.

    It is intended to be used with :obj:`~pyrogram.Client.`.

    Parameters:
        chat_id (``int``):
            Unique identifier (int) or username (str) of the target chat.
    
    """

    def __init__(self, chat_id: int):
        super().__init__()

        self.chat_id = chat_id
        self._raw = raw.types.PaidReactionPrivacyPeer



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.


import pyrogram
from pyrogram import types, raw, utils

from ..object import Object


class StoryArea(Object):
    """This object describes a clickable area on a story media.

    Parameters:
        position (:obj:`~pyrogram.types.StoryAreaPosition`):
            Position of the area.
        
        type (:obj:`~pyrogram.types.StoryAreaType`):
            Type of the area.

    """

    def __init__(
        self,
        position: "types.StoryAreaPosition" = None,
        type: "types.StoryAreaType" = None,
    ):
        super().__init__()

        self.position = position
        self.type = type

    @staticmethod
    def _parse(
        client: "pyrogram.Client",
        area: "raw.base.MediaArea",
    ) -> "StoryArea":
        story_area_type = None
        if isinstance(area, raw.types.MediaAreaVenue):
            # coordinates:MediaAreaCoordinates geo:GeoPoint title:string address:string provider:string venue_id:string venue_type:string
            story_area_type = area
        if isinstance(area, raw.types.MediaAreaGeoPoint):
            story_area_type = types.StoryAreaTypeLocation(
                latitude=area.geo.lat,
                longitude=area.geo.long,
                horizontal_accuracy=area.geo.accuracy_radius,
                address=types.LocationAddress(
                    country_code=area.address.country_iso2,
                    state=area.address.state,
                    city=area.address.city,
                    street=area.address.street,
                ) if area.address else None
            )
        if isinstance(area, raw.types.MediaAreaSuggestedReaction):
            story_area_type = types.StoryAreaTypeSuggestedReaction(
                reaction_type=types.ReactionType._parse(client, area.reaction),
                is_dark=area.dark,
                is_flipped=area.flipped
            )
        if isinstance(area, raw.types.MediaAreaChannelPost):
            story_area_type = types.StoryAreaTypeMessage(
                chat_id=utils.get_channel_id(area.channel_id),
                message_id=area.msg_id
            )
        if isinstance(area, raw.types.MediaAreaUrl):
            story_area_type = types.StoryAreaTypeLink(
                url=area.url
            )
        if isinstance(area, raw.types.MediaAreaWeather):
            story_area_type = types.StoryAreaTypeWeather(
                temperature=area.temperature_c,
                emoji=area.emoji,
                background_color=area.color
            )
        if isinstance(area, raw.types.MediaAreaStarGift):
            story_area_type = types.StoryAreaTypeUniqueGift(
                name=area.slug
            )
        return StoryArea(
            position=types.StoryAreaPosition._parse(area.coordinates),
            type=story_area_type,
        )

    def write(self, client: "pyrogram.Client"):
        coordinates = self.position.write()
        return self.type.write(client, coordinates)



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import pyrogram

from pyrogram import raw, types
from ..object import Object


class PaidMessagePriceChanged(Object):
    """Describes a service message about a change in the price of paid messages within a chat.

    Parameters:
        paid_message_star_count (``int``):
            The new number of Telegram Stars that must be paid by non-administrator users of the supergroup chat for each sent message.

    """

    def __init__(
        self,
        *,
        client: "pyrogram.Client" = None,
        paid_message_star_count: int = None
    ):
        super().__init__(client)

        self.paid_message_star_count = paid_message_star_count


    @staticmethod
    def _parse_action(
        client,
        action: "raw.types.MessageActionPaidMessagesPrice"
    ) -> "PaidMessagePriceChanged":
        if isinstance(action, raw.types.MessageActionPaidMessagesPrice):
            return PaidMessagePriceChanged(
                client=client,
                paid_message_star_count=action.stars
            )



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import logging
from datetime import datetime

import pyrogram
from pyrogram import raw, utils
from pyrogram import types
from pyrogram.file_id import FileId, FileType, FileUniqueId, FileUniqueType, ThumbnailSource
from ..object import Object

log = logging.getLogger(__name__)


class Photo(Object):
    """A Photo.

    Parameters:
        sizes (List of :obj:`~pyrogram.types.Thumbnail`):
            Available variants of the photo, in different sizes.

        date (:py:obj:`~datetime.datetime`):
            Date the photo was sent.

        ttl_seconds (``int``, *optional*):
            Time-to-live seconds, for secret photos.

        thumbs (List of :obj:`~pyrogram.types.Thumbnail`, *optional*):
            Available thumbnails of this photo.

    """

    def __init__(
        self,
        *,
        client: "pyrogram.Client" = None,
        sizes: list["types.Thumbnail"],
        date: datetime,
        ttl_seconds: int = None,
        has_spoiler: bool = None,
        thumbs: list["types.Thumbnail"] = None
    ):
        super().__init__(client)

        self.sizes = sizes
        self.date = date
        self.ttl_seconds = ttl_seconds
        self.has_spoiler = has_spoiler
        self.thumbs = thumbs

    @staticmethod
    def _parse(
        client,
        photo: "raw.types.Photo",
        ttl_seconds: int = None,
        has_spoiler: bool = None,
    ) -> "Photo":
        if isinstance(photo, raw.types.Photo):
            photos: list[raw.types.PhotoSize] = []

            for p in photo.sizes:
                if isinstance(p, raw.types.PhotoSize):
                    photos.append(p)

                if isinstance(p, raw.types.PhotoSizeProgressive):
                    photos.append(
                        raw.types.PhotoSize(
                            type=p.type,
                            w=p.w,
                            h=p.h,
                            size=max(p.sizes)
                        )
                    )

            photos.sort(key=lambda p: p.size)

            return Photo(
                sizes=[
                    types.Thumbnail(
                        file_id=FileId(
                            file_type=FileType.PHOTO,
                            dc_id=photo.dc_id,
                            media_id=photo.id,
                            access_hash=photo.access_hash,
                            file_reference=photo.file_reference,
                            thumbnail_source=ThumbnailSource.THUMBNAIL,
                            thumbnail_file_type=FileType.PHOTO,
                            thumbnail_size=main.type,
                            volume_id=0,
                            local_id=0
                        ).encode(),
                        file_unique_id=FileUniqueId(
                            file_unique_type=FileUniqueType.DOCUMENT,
                            media_id=photo.id
                        ).encode(),
                        width=main.w,
                        height=main.h,
                        file_size=main.size,
                    )
                    for main in photos
                ],
                date=utils.timestamp_to_datetime(photo.date),
                ttl_seconds=ttl_seconds,
                has_spoiler=has_spoiler,
                thumbs=types.Thumbnail._parse(client, photo),
                client=client
            )

    @property
    def file_id(self) -> str:
        log.warning(
            "This property is deprecated. "
            "Please use sizes instead"
        )
        if len(self.sizes) > 0:
            return self.sizes[-1].file_id
        return None

    @property
    def file_unique_id(self) -> str:
        log.warning(
            "This property is deprecated. "
            "Please use sizes instead"
        )
        if len(self.sizes) > 0:
            return self.sizes[-1].file_unique_id
        return None

    @property
    def width(self) -> int:
        log.warning(
            "This property is deprecated. "
            "Please use sizes instead"
        )
        if len(self.sizes) > 0:
            return self.sizes[-1].width
        return None

    @property
    def height(self) -> int:
        log.warning(
            "This property is deprecated. "
            "Please use sizes instead"
        )
        if len(self.sizes) > 0:
            return self.sizes[-1].height
        return None

    @property
    def file_size(self) -> int:
        log.warning(
            "This property is deprecated. "
            "Please use sizes instead"
        )
        if len(self.sizes) > 0:
            return self.sizes[-1].file_size
        return None



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import io
from typing import Optional, Union

from .input_paid_media import InputPaidMedia
from ... import enums


class InputPaidMediaVideo(InputPaidMedia):
    """The paid media to send is a video.

    It is intended to be used with :obj:`~pyrogram.Client.send_paid_media`.

    Parameters:
        media (``str`` | :obj:`io.BytesIO`):
            File to send.
            Pass a file_id as string to send a video that exists on the Telegram servers or
            pass a file path as string to upload a new video that exists on your local machine or
            pass a binary file-like object with its attribute “.name” set for in-memory uploads or
            pass an HTTP URL as a string for Telegram to get a video from the Internet.

        thumbnail (``str`` | :obj:`io.BytesIO`):
            Thumbnail of the video sent.
            The thumbnail should be in JPEG format and less than 200 KB in size.
            A thumbnail's width and height should not exceed 320 pixels.
            Thumbnails can't be reused and can be only uploaded as a new file.

        width (``int``, *optional*):
            Video width.

        height (``int``, *optional*):
            Video height.

        duration (``int``, *optional*):
            Video duration.

        supports_streaming (``bool``, *optional*):
            Pass True, if the uploaded video is suitable for streaming.

        cover (``str`` | :obj:`io.BytesIO`, *optional*):
            Cover for the video in the message. pass None to skip cover uploading.
        
        start_timestamp (``int``, *optional*):
            Timestamp from which the video playing must start, in seconds.

    """

    def __init__(
        self,
        media: Union[str, "io.BytesIO"],
        thumbnail: Union[str, "io.BytesIO"] = None,
        width: int = 0,
        height: int = 0,
        duration: int = 0,
        supports_streaming: bool = True,
        cover: Optional[Union[str, "io.BytesIO"]] = None,
        start_timestamp: int = None
    ):
        super().__init__(media)

        self.thumbnail = thumbnail
        self.width = width
        self.height = height
        self.duration = duration
        self.supports_streaming = supports_streaming
        self.cover = cover
        self.start_timestamp = start_timestamp



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.


import pyrogram
from pyrogram import types

from .story_origin import StoryOrigin


class StoryOriginPublicStory(StoryOrigin):
    """The original story was a public story that was posted by a known chat.

    Parameters:
        chat (:obj:`~pyrogram.types.Chat`):
            Identifier of the chat that posted original story.
        
        story_id (``int``):
            Story identifier of the original story.

    """

    def __init__(
        self,
        *,
        chat: "types.Chat" = None,
        story_id: int = None
    ):
        super().__init__()

        self.chat = chat
        self.story_id = story_id



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from typing import Optional

import pyrogram
from pyrogram import raw

from ..object import Object


class BusinessBotRights(Object):
    """Represents the rights of a business bot.

    Parameters:
        can_reply (``bool``, *optional*):
            True, if the bot can send and edit messages in the private chats that had incoming messages in the last 24 hours.

        can_read_messages (``bool``, *optional*):
            True, if the bot can mark incoming private messages as read.

        can_delete_sent_messages (``bool``, *optional*):
            True, if the bot can delete messages sent by the bot.

        can_delete_all_messages (``bool``, *optional*):
            True, if the bot can delete all private messages in managed chats.

        can_edit_name (``bool``, *optional*):
            True, if the bot can edit the first and last name of the business account.

        can_edit_bio (``bool``, *optional*):
            True, if the bot can edit the bio of the business account.

        can_edit_profile_photo (``bool``, *optional*):
            True, if the bot can edit the profile photo of the business account.

        can_edit_username (``bool``, *optional*):
            True, if the bot can edit the username of the business account.

        can_change_gift_settings (``bool``, *optional*):
            True, if the bot can change the privacy settings pertaining to gifts for the business account.

        can_view_gifts_and_stars (``bool``, *optional*):
            True, if the bot can view gifts and the amount of Telegram Stars owned by the business account.

        can_convert_gifts_to_stars (``bool``, *optional*):
            True, if the bot can convert regular gifts owned by the business account to Telegram Stars.
        
        can_transfer_and_upgrade_gifts (``bool``, *optional*):
            True, if the bot can transfer and upgrade gifts owned by the business account.

        can_transfer_stars (``bool``, *optional*):
            True, if the bot can transfer Telegram Stars received by the business account to its own account, or use them to upgrade and transfer gifts.

        can_manage_stories (``bool``, *optional*):
            True, if the bot can post, edit and delete stories on behalf of the business account.

    """

    def __init__(
        self,
        *,
        can_reply: Optional[bool] = None,
        can_read_messages: Optional[bool] = None,
        can_delete_sent_messages: Optional[bool] = None,
        can_delete_all_messages: Optional[bool] = None,
        can_edit_name: Optional[bool] = None,
        can_edit_bio: Optional[bool] = None,
        can_edit_profile_photo: Optional[bool] = None,
        can_edit_username: Optional[bool] = None,
        can_change_gift_settings: Optional[bool] = None,
        can_view_gifts_and_stars: Optional[bool] = None,
        can_convert_gifts_to_stars: Optional[bool] = None,
        can_transfer_and_upgrade_gifts: Optional[bool] = None,
        can_transfer_stars: Optional[bool] = None,
        can_manage_stories: Optional[bool] = None,
    ):
        super().__init__()

        self.can_reply = can_reply
        self.can_read_messages = can_read_messages
        self.can_delete_sent_messages = can_delete_sent_messages
        self.can_delete_all_messages = can_delete_all_messages
        self.can_edit_name = can_edit_name
        self.can_edit_bio = can_edit_bio
        self.can_edit_profile_photo = can_edit_profile_photo
        self.can_edit_username = can_edit_username
        self.can_change_gift_settings = can_change_gift_settings
        self.can_view_gifts_and_stars = can_view_gifts_and_stars
        self.can_convert_gifts_to_stars = can_convert_gifts_to_stars
        self.can_transfer_and_upgrade_gifts = can_transfer_and_upgrade_gifts
        self.can_transfer_stars = can_transfer_stars
        self.can_manage_stories = can_manage_stories


    @staticmethod
    def _parse(
        client,
        business_bot_rights: "raw.types.BusinessBotRights"
    ) -> "BusinessBotRights":
        if not business_bot_rights:
            return None
        return BusinessBotRights(
            can_reply=business_bot_rights.reply,
            can_read_messages=business_bot_rights.read_messages,
            can_delete_sent_messages=business_bot_rights.delete_sent_messages,
            can_delete_all_messages=business_bot_rights.delete_received_messages,
            can_edit_name=business_bot_rights.edit_name,
            can_edit_bio=business_bot_rights.edit_bio,
            can_edit_profile_photo=business_bot_rights.edit_profile_photo,
            can_edit_username=business_bot_rights.edit_username,
            can_change_gift_settings=business_bot_rights.change_gift_settings,
            can_view_gifts_and_stars=business_bot_rights.view_gifts,
            can_convert_gifts_to_stars=business_bot_rights.sell_gifts,
            can_transfer_and_upgrade_gifts=business_bot_rights.transfer_and_upgrade_gifts,
            can_transfer_stars=business_bot_rights.transfer_stars,
            can_manage_stories=business_bot_rights.manage_stories,
        )



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from typing import Optional

import pyrogram
from pyrogram import enums, raw, types, utils
from ..object import Object
from ..update import Update
from .message import Str


class PollAnswer(Object, Update):
    """This object represents an answer of a user in a non-anonymous poll.

    Parameters:
        poll_id (``str``):
            Unique poll identifier.

        voter_chat (:obj:`~pyrogram.types.Chat`, *optional*):
            The chat that changed the answer to the poll, if the voter is anonymous.

        user (:obj:`~pyrogram.types.User`, *optional*):
            The user that changed the answer to the poll, if the voter isn't anonymous.

        option_ids (List of ``int``):
            0-based identifiers of chosen answer options. May be empty if the vote was retracted.

    """

    def __init__(
        self,
        *,
        client: "pyrogram.Client" = None,
        poll_id: str,
        option_ids: list[int],
        user: Optional["types.User"] = None,
        voter_chat: Optional["types.Chat"] = None,
    ):
        super().__init__(client)

        self.poll_id = poll_id
        self.option_ids = option_ids
        self.user = user
        self.voter_chat = voter_chat

    @staticmethod
    def _parse_update(
        client,
        update: "raw.types.UpdateMessagePollVote",
        users: dict,
        chats: dict,
    ):
        if isinstance(update, raw.types.UpdateMessagePollVote):
            user = None
            voter_chat = None
            if isinstance(update.peer, raw.types.PeerUser):
                user = types.Chat._parse_user_chat(client, users[update.peer.user_id])

            elif isinstance(update.peer, raw.types.PeerChat):
                voter_chat = types.Chat._parse_chat_chat(client, chats[update.peer.chat_id])

            else:
                voter_chat = types.Chat._parse_channel_chat(client, chats[update.peer.channel_id])

            return PollAnswer(
                poll_id=str(update.poll_id),
                option_ids=[
                    "{:0>2x}".format(option[0])
                    for option in update.options
                ],
                user=user,
                voter_chat=voter_chat,
                client=client
            )



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present <https://github.com/TelegramPlayGround>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

import pyrogram
from pyrogram import raw

from .story_area_type import StoryAreaType


class StoryAreaTypeUniqueGift(StoryAreaType):
    """This object describes a story area pointing to a unique gift. Currently, a story can have at most 1 unique gift area.

    Parameters:
        name (``str``):
            Unique name of the gift.

    """

    def __init__(
        self,
        name: str = None,
    ):
        super().__init__()

        self.name = name

    async def write(
        self,
        client: "pyrogram.Client",
        coordinates: "raw.types.MediaAreaCoordinates"
    ):
        return raw.types.MediaAreaStarGift(
            coordinates=coordinates,
            slug=self.name
        )



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from asyncio import sleep
from typing import Union, Optional, AsyncGenerator

import pyrogram
from pyrogram import raw, types


class GetReceivedGifts:
    async def get_received_gifts(
        self: "pyrogram.Client",
        owner_id: Union[int, str],
        offset: str = "",
        limit: int = 0,
        exclude_unsaved: bool = None,
        exclude_saved: bool = None,
        exclude_unlimited: bool = None,
        exclude_limited: bool = None,
        exclude_upgraded: bool = None,
        sort_by_price: bool = None
    ) -> Optional[AsyncGenerator["types.ReceivedGift", None]]:
        """Returns gifts received by the given user or chat.

        .. include:: /_includes/usable-by/users.rst

        Parameters:
            owner_id (``int`` | ``str``):
                Unique identifier (int) or username (str) of the target chat.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".
                For a contact that exists in your Telegram address book you can use his phone number (str).

            offset (``str``, *optional*):
                Offset of the first entry to return as received from the previous request; use empty string to get the first chunk of results.

            limit (``int``, *optional*):
                The maximum number of gifts to be returned; must be positive and can't be greater than 100. For optimal performance, the number of returned objects is chosen by Telegram Server and can be smaller than the specified limit.

            exclude_unsaved (``bool``, *optional*):
                Pass True to exclude gifts that aren't saved to the chat's profile page. Always True for gifts received by other users and channel chats without ``can_post_messages`` administrator right.

            exclude_saved (``bool``, *optional*):
                Pass True to exclude gifts that are saved to the chat's profile page. Always False for gifts received by other users and channel chats without ``can_post_messages`` administrator right.

            exclude_unlimited (``bool``, *optional*):
                Pass True to exclude gifts that can be purchased unlimited number of times.

            exclude_limited (``bool``, *optional*):
                Pass True to exclude gifts that can be purchased limited number of times.

            exclude_upgraded (``bool``, *optional*):
                Pass True to exclude upgraded gifts.

            sort_by_price (``bool``, *optional*):
                Pass True to sort results by gift price instead of send date.

        Returns:
            ``Generator``: A generator yielding :obj:`~pyrogram.types.ReceivedGift` objects.

        Example:
            .. code-block:: python

                async for received_gift in app.get_received_gifts(owner_id):
                    print(received_gift)
        """
        peer = await self.resolve_peer(owner_id)

        current = 0
        total = abs(limit) or (1 << 31) - 1
        limit = min(100, total)

        while True:
            r = await self.invoke(
                raw.functions.payments.GetSavedStarGifts(
                    peer=peer,
                    offset=offset,
                    limit=limit,
                    exclude_unsaved=exclude_unsaved,
                    exclude_saved=exclude_saved,
                    exclude_unlimited=exclude_unlimited,
                    exclude_limited=exclude_limited,
                    exclude_unique=exclude_upgraded,
                    sort_by_value=sort_by_price,
                    # collection_id=
                ),
                sleep_threshold=60
            )

            users = {u.id: u for u in r.users}
            chats = {c.id: c for c in r.chats}

            received_gifts = [
                await types.ReceivedGift._parse(self, gift, users, chats)
                for gift in r.gifts
            ]

            if not received_gifts:
                return

            for received_gift in received_gifts:
                await sleep(0)
                yield received_gift

                current += 1

                if current >= total:
                    return

            offset = r.next_offset

            if not offset:
                return



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from typing import Union

import pyrogram
from pyrogram import raw, types

from .inline_session import get_session


class StopPoll:
    async def stop_poll(
        self: "pyrogram.Client",
        chat_id: Union[int, str],
        message_id: int,
        reply_markup: "types.InlineKeyboardMarkup" = None,
        business_connection_id: str = None
    ) -> "types.Poll":
        """Stop a poll which was sent by you.

        Stopped polls can't be reopened and nobody will be able to vote in it anymore.

        .. include:: /_includes/usable-by/users-bots.rst

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier (int) or username (str) of the target chat.
                For your personal cloud (Saved Messages) you can simply use "me" or "self".
                For a contact that exists in your Telegram address book you can use his phone number (str).

            message_id (``int``):
                Identifier of the original message with the poll.

            reply_markup (:obj:`~pyrogram.types.InlineKeyboardMarkup`, *optional*):
                An InlineKeyboardMarkup object.

            business_connection_id (``str``, *optional*):
                Unique identifier of the business connection on behalf of which the message to be edited was sent

        Returns:
            :obj:`~pyrogram.types.Poll`: On success, the stopped poll with the final results is returned.

        Example:
            .. code-block:: python

                await app.stop_poll(chat_id, message_id)
        """
        poll = (await self.get_messages(
            chat_id=chat_id,
            message_ids=message_id
        )).poll
        # TODO
        rpc = raw.functions.messages.EditMessage(
            peer=await self.resolve_peer(chat_id),
            id=message_id,
            media=raw.types.InputMediaPoll(
                poll=raw.types.Poll(
                    id=int(poll.id),
                    closed=True,
                    question=raw.types.TextWithEntities(text="", entities=[]),
                    answers=[]
                )
            ),
            reply_markup=await reply_markup.write(self) if reply_markup else None
        )
        session = None
        business_connection = None
        if business_connection_id:
            business_connection = self.business_user_connection_cache[business_connection_id]
            if business_connection is None:
                business_connection = await self.get_business_connection(business_connection_id)
            session = await get_session(
                self,
                business_connection._raw.connection.dc_id
            )
        if business_connection_id:
            r = await session.invoke(
                raw.functions.InvokeWithBusinessConnection(
                    query=rpc,
                    connection_id=business_connection_id
                )
            )
            # await session.stop()
        else:
            r = await self.invoke(rpc)

        return types.Poll._parse(self, r.updates[0])
